# Requests
from Doraemon.Requests import chrome_dora
from Doraemon.Requests import requests_dora
from Doraemon.Requests import proxies_dora

# Crawlers
from Doraemon.Crawlers import google_KG
from Doraemon.Crawlers import google_translator
from Doraemon.Crawlers import dianping
from Doraemon.Crawlers import whois
from Doraemon.Crawlers import qq_music_crawler_by_album, qq_music_crawler_by_area
from Doraemon.Crawlers import investigation

# Other
from Doraemon.Others import domain2ip, ext_apis
